@@ -19,10 +19,9 @@ import pytest
 import pytestqt
 from qtpy.QtCore import Qt
 from qtpy.QtGui import QTextCursor
-from rtree import index
 
 # Local imports
-from spyder.config.base import running_in_ci
+from spyder.config.base import running_in_ci, running_in_ci_with_conda
 from spyder.config.utils import is_anaconda
 from spyder.plugins.completion.api import (
     CompletionRequestTypes, CompletionItemKind)
@@ -532,7 +531,6 @@ def test_completions(completions_codeeditor, qtbot):
         # This is a regression test for spyder-ide/spyder#11600
         pass
 
-
     # enter for new line
     qtbot.keyPress(code_editor, Qt.Key_Enter, delay=300)
 
@@ -1168,7 +1166,11 @@ def spam():
 @pytest.mark.slow
 @pytest.mark.order(1)
 @pytest.mark.skipif(not is_anaconda(), reason='Requires conda to be installed')
-@pytest.mark.skipif(not running_in_ci(), reason='Run tests only on CI.')
+@pytest.mark.skipif(not running_in_ci_with_conda(), reason='Run tests only on CI with conda.')
+@pytest.mark.skipif(running_in_ci() and sys.platform.startswith('linux'),
+                    reason="Quite flaky with Linux on CI")
+@pytest.mark.skipif(running_in_ci() and sys.platform == 'darwin',
+                    reason="Quite flaky with MacOS on CI")
 @flaky(max_runs=5)
 def test_completions_environment(completions_codeeditor, qtbot, tmpdir):
     """Exercise code completion when adding extra paths."""
@@ -1194,6 +1196,7 @@ def test_completions_environment(completions_codeeditor, qtbot, tmpdir):
     # Set environment
     set_executable_config_helper()
     completion_plugin.after_configuration_update([])
+    qtbot.wait(2000)
 
     qtbot.keyClicks(code_editor, 'import flas')
     qtbot.keyPress(code_editor, Qt.Key_Tab)
@@ -1203,11 +1206,12 @@ def test_completions_environment(completions_codeeditor, qtbot, tmpdir):
     # Reset extra paths
     set_executable_config_helper(py_exe)
     completion_plugin.after_configuration_update([])
+    qtbot.wait(2000)
 
     code_editor.set_text('')
-    qtbot.keyClicks(code_editor, 'import flas')
     with qtbot.waitSignal(completion.sig_show_completions,
                           timeout=10000) as sig:
+        qtbot.keyClicks(code_editor, 'import flas')
         qtbot.keyPress(code_editor, Qt.Key_Tab)
 
     qtbot.keyPress(completion, Qt.Key_Tab)
