@@ -13,7 +13,7 @@ import os
 import os.path as osp
 
 # Third party imports
-from qtpy.QtCore import Signal
+from qtpy.QtCore import Signal, Slot
 
 # Local imports
 from spyder.api.plugins import Plugins, SpyderDockablePlugin
@@ -244,7 +244,6 @@ class IPythonConsole(SpyderDockablePlugin):
         widget.sig_help_requested.connect(self.sig_help_requested)
         widget.sig_current_directory_changed.connect(
             self.sig_current_directory_changed)
-        widget.sig_exception_occurred.connect(self.sig_exception_occurred)
 
         # Update kernels if python path is changed
         self.main.sig_pythonpath_changed.connect(self.update_path)
@@ -319,7 +318,7 @@ class IPythonConsole(SpyderDockablePlugin):
 
         # Connect Editor debug action with Console
         self.sig_pdb_state_changed.connect(editor.update_pdb_state)
-        editor.exec_in_extconsole.connect(self.execute_code_and_focus_editor)
+        editor.exec_in_extconsole.connect(self.run_selection)
         editor.sig_file_debug_message_requested.connect(
             self.print_debug_file_msg)
 
@@ -329,6 +328,12 @@ class IPythonConsole(SpyderDockablePlugin):
         projects.sig_project_loaded.connect(self._on_project_loaded)
         projects.sig_project_closed.connect(self._on_project_closed)
 
+    @on_plugin_available(plugin=Plugins.WorkingDirectory)
+    def on_working_directory_available(self):
+        working_directory = self.get_plugin(Plugins.WorkingDirectory)
+        working_directory.sig_current_directory_changed.connect(
+            self.save_working_directory)
+
     @on_plugin_teardown(plugin=Plugins.Preferences)
     def on_preferences_teardown(self):
         # Register conf page
@@ -360,8 +365,7 @@ class IPythonConsole(SpyderDockablePlugin):
 
         # Connect Editor debug action with Console
         self.sig_pdb_state_changed.disconnect(editor.update_pdb_state)
-        editor.exec_in_extconsole.disconnect(
-            self.execute_code_and_focus_editor)
+        editor.exec_in_extconsole.disconnect(self.run_selection)
         editor.sig_file_debug_message_requested.disconnect(
             self.print_debug_file_msg)
 
@@ -371,6 +375,12 @@ class IPythonConsole(SpyderDockablePlugin):
         projects.sig_project_loaded.disconnect(self._on_project_loaded)
         projects.sig_project_closed.disconnect(self._on_project_closed)
 
+    @on_plugin_teardown(plugin=Plugins.WorkingDirectory)
+    def on_working_directory_teardown(self):
+        working_directory = self.get_plugin(Plugins.WorkingDirectory)
+        working_directory.sig_current_directory_changed.disconnect(
+            self.save_working_directory)
+
     def update_font(self):
         """Update font from Preferences"""
         font = self.get_font()
@@ -380,26 +390,17 @@ class IPythonConsole(SpyderDockablePlugin):
     def on_close(self, cancelable=False):
         """Perform actions when plugin is closed"""
         self.get_widget().mainwindow_close = True
-        return self.get_widget().close_clients()
+        return self.get_widget().close_all_clients()
 
     def on_mainwindow_visible(self):
         self.create_new_client(give_focus=False)
 
-        # Raise plugin the first time Spyder starts
-        if self.get_conf('show_first_time', default=True):
-            self.dockwidget.raise_()
-            self.set_conf('show_first_time', False)
-
     # ---- Private methods
     # -------------------------------------------------------------------------
     def _load_file_in_editor(self, fname, lineno, word, processevents):
         editor = self.get_plugin(Plugins.Editor)
         editor.load(fname, lineno, word, processevents=processevents)
 
-    def _switch_to_editor(self):
-        editor = self.get_plugin(Plugins.Editor)
-        editor.switch_to_plugin()
-
     def _on_project_loaded(self):
         projects = self.get_plugin(Plugins.Projects)
         self.get_widget().update_active_project_path(
@@ -425,12 +426,6 @@ class IPythonConsole(SpyderDockablePlugin):
                     except Exception:
                         pass
 
-    def _get_working_directory(self):
-        """Get current working directory from its plugin."""
-        workdir = self.get_plugin(Plugins.WorkingDirectory)
-        if workdir:
-            return workdir.get_workdir()
-
     # ---- Public API
     # -------------------------------------------------------------------------
 
@@ -452,7 +447,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().register_spyder_kernel_call_handler(
             handler_id, handler)
@@ -470,7 +464,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().unregister_spyder_kernel_call_handler(handler_id)
 
@@ -491,6 +484,23 @@ class IPythonConsole(SpyderDockablePlugin):
         """Return the shellwidget of the current client"""
         return self.get_widget().get_current_shellwidget()
 
+    def rename_client_tab(self, client, given_name):
+        """
+        Rename a client's tab.
+
+        Parameters
+        ----------
+        client: spyder.plugins.ipythonconsole.widgets.client.ClientWidget
+            Client to rename.
+        given_name: str
+            New name to be given to the client's tab.
+
+        Returns
+        -------
+        None.
+        """
+        self.get_widget().rename_client_tab(client, given_name)
+
     def create_new_client(self, give_focus=True, filename='', is_cython=False,
                           is_pylab=False, is_sympy=False, given_name=None):
         """
@@ -519,7 +529,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().create_new_client(
             give_focus=give_focus,
@@ -543,10 +552,36 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().create_client_for_file(filename, is_cython=is_cython)
 
+    def create_client_for_kernel(self, connection_file, hostname=None,
+                                 sshkey=None, password=None):
+        """
+        Create a client connected to an existing kernel.
+
+        Parameters
+        ----------
+        connection_file: str
+            Json file that has the kernel's connection info.
+        hostname: str, optional
+            Name or IP address of the remote machine where the kernel was
+            started. When this is provided, it's also necessary to pass either
+            the ``sshkey`` or ``password`` arguments.
+        sshkey: str, optional
+            SSH key file to connect to the remote machine where the kernel is
+            running.
+        password: str, optional
+            Password to authenticate to the remote machine where the kernel is
+            running.
+
+        Returns
+        -------
+        None.
+        """
+        self.get_widget().create_client_for_kernel(
+            connection_file, hostname, sshkey, password)
+
     def get_client_for_file(self, filename):
         """Get client associated with a given file name."""
         return self.get_widget().get_client_for_file(filename)
@@ -568,8 +603,10 @@ class IPythonConsole(SpyderDockablePlugin):
                                        ask_recursive=ask_recursive)
 
     # ---- For execution and debugging
-    def run_script(self, filename, wdir, args, debug, post_mortem,
-                   current_client, clear_variables, console_namespace):
+    def run_script(self, filename, wdir, args='', debug=False,
+                   post_mortem=False, current_client=True,
+                   clear_variables=False, console_namespace=False,
+                   focus_to_editor=True):
         """
         Run script in current or dedicated client.
 
@@ -579,28 +616,30 @@ class IPythonConsole(SpyderDockablePlugin):
             Path to file that will be run.
         wdir : str
             Working directory from where the file should be run.
-        args : str
+        args : str, optional
             Arguments defined to run the file.
-        debug : bool
+        debug : bool, optional
             True if the run if for debugging the file,
             False for just running it.
-        post_mortem : bool
+        post_mortem : bool, optional
             True if in case of error the execution should enter in
             post-mortem mode, False otherwise.
-        current_client : bool
+        current_client : bool, optional
             True if the execution should be done in the current client,
             False if the execution needs to be done in a dedicated client.
-        clear_variables : bool
+        clear_variables : bool, optional
             True if all the variables should be removed before execution,
             False otherwise.
-        console_namespace : bool
+        console_namespace : bool, optional
             True if the console namespace should be used, False otherwise.
+        focus_to_editor: bool, optional
+            Leave focus in the editor after execution.
 
         Returns
         -------
         None.
-
         """
+        self.sig_unmaximize_plugin_requested.emit()
         self.get_widget().run_script(
             filename,
             wdir,
@@ -609,7 +648,8 @@ class IPythonConsole(SpyderDockablePlugin):
             post_mortem,
             current_client,
             clear_variables,
-            console_namespace)
+            console_namespace,
+            focus_to_editor)
 
     def run_cell(self, code, cell_name, filename, run_cell_copy,
                  focus_to_editor, function='runcell'):
@@ -639,8 +679,8 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
+        self.sig_unmaximize_plugin_requested.emit()
         self.get_widget().run_cell(
             code, cell_name, filename, run_cell_copy, focus_to_editor,
             function=function)
@@ -669,8 +709,8 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
+        self.sig_unmaximize_plugin_requested.emit()
         self.get_widget().debug_cell(code, cell_name, filename, run_cell_copy,
                                      focus_to_editor)
 
@@ -692,26 +732,20 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().execute_code(
             lines,
             current_client=current_client,
             clear_variables=clear_variables)
 
-    def execute_code_and_focus_editor(self, lines, focus_to_editor=True):
-        """
-        Execute lines in IPython console and eventually set focus
-        to the Editor.
-        """
-        self.execute_code(lines)
-        if focus_to_editor and self.get_plugin(Plugins.Editor):
-            self._switch_to_editor()
-        else:
-            self.switch_to_plugin()
+    def run_selection(self, lines, focus_to_editor=True):
+        """Execute selected lines in the current console."""
+        self.sig_unmaximize_plugin_requested.emit()
+        self.get_widget().execute_code(lines, set_focus=not focus_to_editor)
 
     def stop_debugging(self):
         """Stop debugging in the current console."""
+        self.sig_unmaximize_plugin_requested.emit()
         self.get_widget().stop_debugging()
 
     def get_pdb_state(self):
@@ -722,7 +756,7 @@ class IPythonConsole(SpyderDockablePlugin):
         """Get last pdb step of the current console."""
         return self.get_widget().get_pdb_last_step()
 
-    def pdb_execute_command(self, command):
+    def pdb_execute_command(self, command, focus_to_editor):
         """
         Send command to the pdb kernel if possible.
 
@@ -730,13 +764,15 @@ class IPythonConsole(SpyderDockablePlugin):
         ----------
         command : str
             Command to execute by the pdb kernel.
+        focus_to_editor: bool
+            Leave focus in editor after the command is executed.
 
         Returns
         -------
         None.
-
         """
-        self.get_widget().pdb_execute_command(command)
+        self.sig_unmaximize_plugin_requested.emit()
+        self.get_widget().pdb_execute_command(command, focus_to_editor)
 
     def print_debug_file_msg(self):
         """
@@ -745,7 +781,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().print_debug_file_msg()
 
@@ -762,13 +797,12 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().set_current_client_working_directory(directory)
 
     def set_working_directory(self, dirname):
         """
-        Set current working directory for the `workingdirectory` and `explorer`
+        Set current working directory in the Working Directory and Files
         plugins.
 
         Parameters
@@ -779,10 +813,21 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().set_working_directory(dirname)
 
+    @Slot(str)
+    def save_working_directory(self, dirname):
+        """
+        Save current working directory on the main widget to start new clients.
+
+        Parameters
+        ----------
+        new_dir: str
+            Path to the new current working directory.
+        """
+        self.get_widget().save_working_directory(dirname)
+
     def update_working_directory(self):
         """Update working directory to console current working directory."""
         self.get_widget().update_working_directory()
@@ -804,7 +849,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().update_path(path_dict, new_path_dict)
 
@@ -828,7 +872,6 @@ class IPythonConsole(SpyderDockablePlugin):
         Returns
         -------
         None.
-
         """
         self.get_widget().restart_kernel()
 
