@@ -13,18 +13,20 @@ import os
 import os.path as osp
 import uuid
 from textwrap import dedent
+from threading import Lock
 
 # Third party imports
 from qtpy.QtCore import Signal, QThread
 from qtpy.QtWidgets import QMessageBox
 from qtpy import QtCore, QtWidgets, QtGui
+from traitlets import observe
 
 # Local imports
 from spyder.config.base import (
     _, is_pynsist, running_in_mac_app, running_under_pytest)
+from spyder.config.gui import get_color_scheme
 from spyder.py3compat import to_text_string
 from spyder.utils.palette import SpyderPalette
-from spyder.utils import encoding
 from spyder.utils.clipboard_helper import CLIPBOARD_HELPER
 from spyder.utils import syntaxhighlighters as sh
 from spyder.plugins.ipythonconsole.utils.style import (
@@ -77,8 +79,36 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
     # For printing internal errors
     sig_exception_occurred = Signal(dict)
 
+    # Class array of shutdown threads
+    shutdown_thread_list = []
+
+    @classmethod
+    def prune_shutdown_thread_list(cls):
+        """Remove shutdown threads."""
+        pruned_shutdown_thread_list = []
+        for t in cls.shutdown_thread_list:
+            try:
+                if t.isRunning():
+                    pruned_shutdown_thread_list.append(t)
+            except RuntimeError:
+                pass
+        cls.shutdown_thread_list = pruned_shutdown_thread_list
+
+    @classmethod
+    def wait_all_shutdown(cls):
+        """Wait for shutdown to finish."""
+        for thread in cls.shutdown_thread_list:
+            if thread.isRunning():
+                try:
+                    thread.kernel_manager._kill_kernel()
+                except Exception:
+                    pass
+                thread.quit()
+                thread.wait()
+        cls.shutdown_thread_list = []
+
     def __init__(self, ipyclient, additional_options, interpreter_versions,
-                 external_kernel, *args, **kw):
+                 is_external_kernel, is_spyder_kernel, handlers, *args, **kw):
         # To override the Qt widget used by RichJupyterWidget
         self.custom_control = ControlWidget
         self.custom_page_control = PageControlWidget
@@ -87,14 +117,15 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         self.spyder_kernel_comm.sig_exception_occurred.connect(
             self.sig_exception_occurred)
         super(ShellWidget, self).__init__(*args, **kw)
-
         self.ipyclient = ipyclient
         self.additional_options = additional_options
         self.interpreter_versions = interpreter_versions
-        self.external_kernel = external_kernel
+        self.is_external_kernel = is_external_kernel
+        self.is_spyder_kernel = is_spyder_kernel
         self._cwd = ''
 
         # Keyboard shortcuts
+        # Registered here to use shellwidget as the parent
         self.shortcuts = self.create_shortcuts()
 
         # Set the color of the matched parentheses here since the qtconsole
@@ -102,24 +133,20 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         # set in the qtconsole constructor. See spyder-ide/spyder#4806.
         self.set_bracket_matcher_color_scheme(self.syntax_style)
 
-        self.shutdown_called = False
+        self.shutting_down = False
         self.kernel_manager = None
         self.kernel_client = None
-        self.shutdown_thread = None
-        handlers = {
+        handlers.update({
             'pdb_state': self.set_pdb_state,
             'pdb_execute': self.pdb_execute,
+            'show_pdb_output': self.show_pdb_output,
             'get_pdb_settings': self.get_pdb_settings,
-            'run_cell': self.handle_run_cell,
-            'cell_count': self.handle_cell_count,
-            'current_filename': self.handle_current_filename,
-            'get_file_code': self.handle_get_file_code,
             'set_debug_state': self.set_debug_state,
             'update_syspath': self.update_syspath,
             'do_where': self.do_where,
             'pdb_input': self.pdb_input,
             'request_interrupt_eventloop': self.request_interrupt_eventloop,
-        }
+        })
         for request_id in handlers:
             self.spyder_kernel_comm.register_call_handler(
                 request_id, handlers[request_id])
@@ -127,57 +154,55 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         self._execute_queue = []
         self.executed.connect(self.pop_execute_queue)
 
-        # Internal kernel are always spyder kernels
-        self._is_spyder_kernel = not external_kernel
-
         # Show a message in our installers to explain users how to use
         # modules that don't come with them.
         self.show_modules_message = is_pynsist() or running_in_mac_app()
-
-    def __del__(self):
-        """Avoid destroying shutdown_thread."""
-        if (self.shutdown_thread is not None
-                and self.shutdown_thread.isRunning()):
-            self.shutdown_thread.wait()
+        self.shutdown_lock = Lock()
 
     # ---- Public API ---------------------------------------------------------
-    def is_spyder_kernel(self):
-        """Is the widget a spyder kernel."""
-        return self._is_spyder_kernel
+    def shutdown_kernel(self):
+        """Shutdown kernel."""
+        with self.shutdown_lock:
+            # Avoid calling shutdown_kernel on the same manager twice
+            # from different threads to avoid crash.
+            if self.kernel_manager.shutting_down:
+                return
+            self.kernel_manager.shutting_down = True
+        try:
+            self.kernel_manager.shutdown_kernel()
+        except Exception:
+            # kernel was externally killed
+            pass
 
-    def shutdown(self):
-        """Shutdown kernel"""
-        self.shutdown_called = True
-        self.spyder_kernel_comm.close()
-        self.kernel_manager.stop_restarter()
+    def shutdown(self, shutdown_kernel=True):
+        """Shutdown connection and kernel."""
+        if self.shutting_down:
+            return
+        self.shutting_down = True
+        if shutdown_kernel:
+            if not self.kernel_manager:
+                return
 
-        self.shutdown_thread = QThread()
-        self.shutdown_thread.run = self.kernel_manager.shutdown_kernel
-        if self.kernel_client is not None:
-            self.shutdown_thread.finished.connect(
-                self.kernel_client.stop_channels)
-        self.shutdown_thread.start()
+            self.interrupt_kernel()
+            if self.kernel_manager:
+                self.kernel_manager.stop_restarter()
+            self.spyder_kernel_comm.close()
+            if self.kernel_client is not None:
+                self.kernel_client.stop_channels()
+            if self.kernel_manager:
+                shutdown_thread = QThread(None)
+                shutdown_thread.kernel_manager = self.kernel_manager
+                shutdown_thread.run = self.shutdown_kernel
+                self.shutdown_thread_list.append(shutdown_thread)
+                shutdown_thread.start()
+        else:
+            self.spyder_kernel_comm.close(shutdown_channel=False)
+            if self.kernel_client is not None:
+                self.kernel_client.stop_channels()
+
+        self.prune_shutdown_thread_list()
         super(ShellWidget, self).shutdown()
 
-    def will_close(self, externally_managed):
-        """
-        Close communication channels with the kernel if shutdown was not
-        called. If the kernel is not externally managed, shutdown the kernel
-        as well.
-        """
-        if not self.shutdown_called and not externally_managed:
-            # Make sure the channels are stopped
-            self.spyder_kernel_comm.close()
-            self.kernel_manager.stop_restarter()
-            self.kernel_manager.shutdown_kernel(now=True)
-            if self.kernel_client is not None:
-                self.kernel_client.stop_channels()
-        if externally_managed:
-            self.spyder_kernel_comm.close()
-            if self.kernel_client is not None:
-                self.kernel_client.stop_channels()
-        super(ShellWidget, self).will_close(externally_managed)
-
     def call_kernel(self, interrupt=False, blocking=False, callback=None,
                     timeout=None, display_error=False):
         """
@@ -214,7 +239,8 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         """Set the kernel client and manager"""
         self.kernel_manager = kernel_manager
         self.kernel_client = kernel_client
-        self.spyder_kernel_comm.open_comm(kernel_client)
+        if self.is_spyder_kernel:
+            self.spyder_kernel_comm.open_comm(kernel_client)
 
         # Redefine the complete method to work while debugging.
         self._redefine_complete_for_dbg(self.kernel_client)
@@ -237,6 +263,11 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         Executes source or the input buffer, possibly prompting for more
         input.
         """
+        # Needed for cases where there is no kernel initialized but
+        # an execution is triggered like when setting initial configs.
+        # See spyder-ide/spyder#16896
+        if self.kernel_client is None:
+            return
         if self._executing:
             self._execute_queue.append((source, hidden, interactive))
             return
@@ -265,8 +296,18 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         else:
             self.silent_exec_method(code)
 
-    def set_cwd(self, dirname):
-        """Set shell current working directory."""
+    def set_cwd(self, dirname, emit_cwd_change=False):
+        """
+        Set shell current working directory.
+
+        Parameters
+        ----------
+        dirname: str
+            Path to the new current working directory.
+        emit_cwd_change: bool
+            Whether to emit a Qt signal that informs other panes in Spyder that
+            the current working directory has changed.
+        """
         if os.name == 'nt':
             # Use normpath instead of replacing '\' with '\\'
             # See spyder-ide/spyder#10785
@@ -275,17 +316,39 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
         if self.ipyclient.hostname is None:
             self.call_kernel(interrupt=self.is_debugging()).set_cwd(dirname)
             self._cwd = dirname
+            if emit_cwd_change:
+                self.sig_working_directory_changed.emit(self._cwd)
+
+    def get_cwd(self):
+        """
+        Get current working directory.
+
+        Notes
+        -----
+        * This doesn't ask the kernel for its working directory. Instead, it
+          returns the last value of it saved here.
+        * We do it for performance reasons because we call this method when
+          switching consoles to update the Working Directory toolbar.
+        """
+        return self._cwd
 
     def update_cwd(self):
-        """Update current working directory in the kernel."""
+        """
+        Update working directory in Spyder after getting its value from the
+        kernel.
+        """
         if self.kernel_client is None:
             return
-        self.call_kernel(callback=self.remote_set_cwd).get_cwd()
+        self.call_kernel(callback=self.on_getting_cwd).get_cwd()
 
-    def remote_set_cwd(self, cwd):
-        """Get current working directory from kernel."""
-        self._cwd = cwd
-        self.sig_working_directory_changed.emit(self._cwd)
+    def on_getting_cwd(self, cwd):
+        """
+        If necessary, notify that the working directory was changed to other
+        plugins.
+        """
+        if cwd != self._cwd:
+            self._cwd = cwd
+            self.sig_working_directory_changed.emit(self._cwd)
 
     def set_bracket_matcher_color_scheme(self, color_scheme):
         """Set color scheme for matched parentheses."""
@@ -352,6 +415,12 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
             interrupt=True,
             blocking=True).get_matplotlib_backend()
 
+    def get_mpl_interactive_backend(self):
+        """Call kernel to get current interactive backend."""
+        return self.call_kernel(
+            interrupt=True,
+            blocking=True).get_mpl_interactive_backend()
+
     def set_matplotlib_backend(self, backend_option, pylab=False):
         """Set matplotlib backend given a backend name."""
         cmd = "get_ipython().kernel.set_matplotlib_backend('{}', {})"
@@ -401,7 +470,8 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget,
 
         banner_parts = [
             'Python %s\n' % py_ver,
-            'Type "copyright", "credits" or "license" for more information.\n\n',
+            'Type "copyright", "credits" or "license" for more information.',
+            '\n\n',
             'IPython %s -- An enhanced Interactive Python.\n' % ipy_ver
         ]
         banner = ''.join(banner_parts)
@@ -550,7 +620,7 @@ the sympy module (e.g. plot)
                 # Fixes spyder-ide/spyder#12689
                 self.refresh_namespacebrowser(interrupt=False)
 
-                if not self.external_kernel:
+                if self.is_spyder_kernel:
                     self.call_kernel().close_all_mpl_figures()
         except AttributeError:
             pass
@@ -570,13 +640,13 @@ the sympy module (e.g. plot)
         """Create shortcuts for ipyconsole."""
         inspect = self.config_shortcut(
             self._control.inspect_current_object,
-            context='Console',
+            context='ipython_console',
             name='Inspect current object',
             parent=self)
 
         clear_console = self.config_shortcut(
             self.clear_console,
-            context='Console',
+            context='ipython_console',
             name='Clear shell',
             parent=self)
 
@@ -600,19 +670,19 @@ the sympy module (e.g. plot)
 
         array_inline = self.config_shortcut(
             self._control.enter_array_inline,
-            context='array_builder',
+            context='ipython_console',
             name='enter array inline',
             parent=self)
 
         array_table = self.config_shortcut(
             self._control.enter_array_table,
-            context='array_builder',
+            context='ipython_console',
             name='enter array table',
             parent=self)
 
         clear_line = self.config_shortcut(
             self.ipyclient.clear_line,
-            context='console',
+            context='ipython_console',
             name='clear line',
             parent=self)
 
@@ -661,11 +731,14 @@ the sympy module (e.g. plot)
         if self.kernel_client is None:
             return
 
-        msg_id = self.kernel_client.execute('', silent=True,
-                                            user_expressions={ local_uuid:code })
+        msg_id = self.kernel_client.execute(
+            '', silent=True,
+            user_expressions={local_uuid: code})
         self._kernel_methods[local_uuid] = code
-        self._request_info['execute'][msg_id] = self._ExecutionRequest(msg_id,
-                                                          'silent_exec_method')
+        self._request_info['execute'][msg_id] = self._ExecutionRequest(
+            msg_id,
+            'silent_exec_method',
+            False)
 
     def handle_exec_method(self, msg):
         """
@@ -687,7 +760,7 @@ the sympy module (e.g. plot)
                     if data is not None and 'text/plain' in data:
                         is_spyder_kernel = data['text/plain']
                         if 'SpyderKernel' in is_spyder_kernel:
-                            self._is_spyder_kernel = True
+                            self.is_spyder_kernel = True
                             self.sig_is_spykernel.emit(self)
 
                 # Remove method after being processed
@@ -700,9 +773,9 @@ the sympy module (e.g. plot)
         """
         calling_mayavi = False
         lines = command.splitlines()
-        for l in lines:
-            if not l.startswith('#'):
-                if 'import mayavi' in l or 'from mayavi' in l:
+        for line in lines:
+            if not line.startswith('#'):
+                if 'import mayavi' in line or 'from mayavi' in line:
                     calling_mayavi = True
                     break
         if calling_mayavi:
@@ -720,7 +793,7 @@ the sympy module (e.g. plot)
         """
         if (command.startswith('%matplotlib') and
                 len(command.splitlines()) == 1):
-            if not 'inline' in command:
+            if 'inline' not in command:
                 self.silent_execute(command)
 
     def append_html_message(self, html, before_prompt=False,
@@ -771,80 +844,6 @@ the sympy module (e.g. plot)
         """
         self._control.insert_horizontal_ruler()
 
-    # ---- Spyder-kernels methods ---------------------------------------------
-    def get_editor(self, filename):
-        """Get editor for filename and set it as the current editor."""
-        editorstack = self.get_editorstack()
-        if editorstack is None:
-            return None
-
-        if not filename:
-            return None
-
-        index = editorstack.has_filename(filename)
-        if index is None:
-            return None
-
-        return editorstack.data[index].editor
-
-    def get_editorstack(self):
-        """Get the current editorstack."""
-        plugin = self.ipyclient.plugin
-        if plugin.main.editor is not None:
-            editor = plugin.main.editor
-            return editor.get_current_editorstack()
-        raise RuntimeError('No editorstack found.')
-
-    def handle_get_file_code(self, filename, save_all=True):
-        """
-        Return the bytes that compose the file.
-
-        Bytes are returned instead of str to support non utf-8 files.
-        """
-        editorstack = self.get_editorstack()
-        if save_all and self.get_conf(
-                'save_all_before_run', default=True, section='editor'):
-            editorstack.save_all(save_new_files=False)
-        editor = self.get_editor(filename)
-
-        if editor is None:
-            # Load it from file instead
-            text, _enc = encoding.read(filename)
-            return text
-
-        return editor.toPlainText()
-
-    def handle_run_cell(self, cell_name, filename):
-        """
-        Get cell code from cell name and file name.
-        """
-        editorstack = self.get_editorstack()
-        editor = self.get_editor(filename)
-
-        if editor is None:
-            raise RuntimeError(
-                "File {} not open in the editor".format(filename))
-
-        editorstack.last_cell_call = (filename, cell_name)
-
-        # The file is open, load code from editor
-        return editor.get_cell_code(cell_name)
-
-    def handle_cell_count(self, filename):
-        """Get number of cells in file to loop."""
-        editor = self.get_editor(filename)
-
-        if editor is None:
-            raise RuntimeError(
-                "File {} not open in the editor".format(filename))
-
-        # The file is open, get cell count from editor
-        return editor.get_cell_count()
-
-    def handle_current_filename(self):
-        """Get the current filename."""
-        return self.get_editorstack().get_current_finfo().filename
-
     # ---- Public methods (overrode by us) ------------------------------------
     def request_restart_kernel(self):
         """Reimplemented to call our own restart mechanism."""
@@ -968,7 +967,7 @@ the sympy module (e.g. plot)
         banner or not
         """
         # Don't change banner for external kernels
-        if self.external_kernel:
+        if self.is_external_kernel:
             return ''
         show_banner_o = self.additional_options['show_banner']
         if show_banner_o:
@@ -984,7 +983,8 @@ the sympy module (e.g. plot)
         super(ShellWidget, self)._handle_kernel_restarted(*args, **kwargs)
         self.sig_kernel_restarted.emit()
 
-    def _syntax_style_changed(self):
+    @observe('syntax_style')
+    def _syntax_style_changed(self, changed=None):
         """Refresh the highlighting with the current syntax style by class."""
         if self._highlighter is None:
             # ignore premature calls
@@ -995,6 +995,17 @@ the sympy module (e.g. plot)
         else:
             self._highlighter.set_style_sheet(self.style_sheet)
 
+    def _get_color(self, color):
+        """
+        Get a color as qtconsole.styles._get_color() would return from
+        a builtin Pygments style.
+        """
+        color_scheme = get_color_scheme(self.syntax_style)
+        return dict(
+            bgcolor=color_scheme['background'],
+            select=color_scheme['background'],
+            fgcolor=color_scheme['normal'][0])[color]
+
     def _prompt_started_hook(self):
         """Emit a signal when the prompt is ready."""
         if not self._reading:
@@ -1025,7 +1036,7 @@ the sympy module (e.g. plot)
                 )
             self.show_modules_message = False
 
-    #---- Qt methods ----------------------------------------------------------
+    # --- Qt methods ----------------------------------------------------------
     def focusInEvent(self, event):
         """Reimplement Qt method to send focus change notification"""
         self.sig_focus_changed.emit()
