@@ -4,9 +4,6 @@
 # Licensed under the terms of the MIT License
 # (see spyder/__init__.py for details)
 
-# Standard library imports
-import sys
-
 # Third party imports
 from qtpy.QtCore import Signal
 from qtpy.QtWidgets import QAction
@@ -43,6 +40,7 @@ class PreferencesContainer(PluginMainContainer):
 
         def _dialog_finished(result_code):
             """Restore preferences dialog instance variable."""
+            self.dialog.disconnect()
             self.dialog = None
 
         if self.dialog is None:
@@ -97,7 +95,8 @@ class PreferencesContainer(PluginMainContainer):
 
     def close_preferences(self):
         """Close preferences"""
-        self.dialog.close()
+        if self.dialog is not None:
+            self.dialog.close()
 
     def show_preferences(self):
         """Show preferences."""
