@@ -35,7 +35,7 @@ from spyder.config.base import get_conf_path, running_in_mac_app
 from spyder.plugins.pylint.utils import get_pylintrc_path
 from spyder.plugins.variableexplorer.widgets.texteditor import TextEditor
 from spyder.utils.icon_manager import ima
-from spyder.utils.misc import getcwd_or_home
+from spyder.utils.misc import getcwd_or_home, get_home_dir
 from spyder.utils.palette import QStylePalette, SpyderPalette
 from spyder.widgets.comboboxes import (PythonModulesComboBox,
                                        is_module_or_package)
@@ -87,7 +87,54 @@ class PylintWidgetToolbarItems:
     Stretcher2 = 'stretcher_2'
 
 
-# --- Widgets
+# ---- Items
+class CategoryItem(QTreeWidgetItem):
+    """
+    Category item for results.
+
+    Notes
+    -----
+    Possible categories are Convention, Refactor, Warning and Error.
+    """
+
+    CATEGORIES = {
+        "Convention": {
+            'translation_string': _("Convention"),
+            'icon': ima.icon("convention")
+        },
+        "Refactor": {
+            'translation_string': _("Refactor"),
+            'icon': ima.icon("refactor")
+        },
+        "Warning": {
+            'translation_string': _("Warning"),
+            'icon': ima.icon("warning")
+        },
+        "Error": {
+            'translation_string': _("Error"),
+            'icon': ima.icon("error")
+        }
+    }
+
+    def __init__(self, parent, category, number_of_messages):
+        # Messages string to append to category.
+        if number_of_messages > 1 or number_of_messages == 0:
+            messages = _('messages')
+        else:
+            messages = _('message')
+
+        # Category title.
+        title = self.CATEGORIES[category]['translation_string']
+        title += f" ({number_of_messages} {messages})"
+
+        super().__init__(parent, [title], QTreeWidgetItem.Type)
+
+        # Set icon
+        icon = self.CATEGORIES[category]['icon']
+        self.setIcon(0, icon)
+
+
+# ---- Widgets
 # ----------------------------------------------------------------------------
 # TODO: display results on 3 columns instead of 1: msg_id, lineno, message
 class ResultsTree(OneColumnTree):
@@ -122,8 +169,14 @@ class ResultsTree(OneColumnTree):
             self.sig_edit_goto_requested.emit(fname, lineno, "")
 
     def clicked(self, item):
-        """Click event"""
-        self.activated(item)
+        """Click event."""
+        if isinstance(item, CategoryItem):
+            if item.isExpanded():
+                self.collapseItem(item)
+            else:
+                self.expandItem(item)
+        else:
+            self.activated(item)
 
     def clear_results(self):
         self.clear()
@@ -135,23 +188,21 @@ class ResultsTree(OneColumnTree):
         self.refresh()
 
     def refresh(self):
-        title = _("Results for ")+self.filename
+        title = _("Results for ") + self.filename
         self.set_title(title)
         self.clear()
         self.data = {}
 
         # Populating tree
         results = (
-            (_("Convention"), ima.icon("convention"), self.results["C:"]),
-            (_("Refactor"), ima.icon("refactor"), self.results["R:"]),
-            (_("Warning"), ima.icon("warning"), self.results["W:"]),
-            (_("Error"), ima.icon("error"), self.results["E:"]),
+            ("Convention", self.results["C:"]),
+            ("Refactor", self.results["R:"]),
+            ("Warning", self.results["W:"]),
+            ("Error", self.results["E:"]),
         )
-        for title, icon, messages in results:
-            title += " (%d message%s)" % (len(messages),
-                                          "s" if len(messages) > 1 else "")
-            title_item = QTreeWidgetItem(self, [title], QTreeWidgetItem.Type)
-            title_item.setIcon(0, icon)
+
+        for category, messages in results:
+            title_item = CategoryItem(self, category, len(messages))
             if not messages:
                 title_item.setDisabled(True)
 
@@ -314,6 +365,13 @@ class PylintWidget(PluginMainWidget):
         processEnvironment = QProcessEnvironment()
         processEnvironment.insert("PYTHONIOENCODING", "utf8")
 
+        # Needed due to changes in Pylint 2.14.0
+        # See spyder-ide/spyder#18175
+        if os.name == 'nt':
+            home_dir = get_home_dir()
+            user_profile = os.environ.get("USERPROFILE", home_dir)
+            processEnvironment.insert("USERPROFILE", user_profile)
+
         # resolve spyder-ide/spyder#14262
         if running_in_mac_app():
             pyhome = os.environ.get("PYTHONHOME")
@@ -384,8 +442,8 @@ class PylintWidget(PluginMainWidget):
         return process is not None and process.state() == QProcess.Running
 
     def _kill_process(self):
-        self._process.kill()
-        self._process.waitForFinished()
+        self._process.close()
+        self._process.waitForFinished(1000)
         self.stop_spinner()
 
     def _update_combobox_history(self):
@@ -414,7 +472,7 @@ class PylintWidget(PluginMainWidget):
             for fname in self.curr_filenames:
                 if _("untitled") not in fname:
                     filename = osp.normpath(fname)
-                    list_save_files.append(fname)
+                    list_save_files.append(filename)
 
             self.curr_filenames = list_save_files[:MAX_HISTORY_ENTRIES]
             self.set_conf("history_filenames", self.curr_filenames)
@@ -549,6 +607,9 @@ class PylintWidget(PluginMainWidget):
 
         self.remove_obsolete_items()
 
+    def on_close(self):
+        self.stop_code_analysis()
+
     # --- Public API
     # ------------------------------------------------------------------------
     @Slot()
