@@ -19,8 +19,10 @@ import re
 import sre_constants
 import sys
 import textwrap
+from pkg_resources import parse_version
 
 # Third party imports
+from qtpy import QT_VERSION
 from qtpy.QtCore import QPoint, QRegularExpression, Qt
 from qtpy.QtGui import QCursor, QTextCursor, QTextDocument
 from qtpy.QtWidgets import QApplication
@@ -29,7 +31,7 @@ from spyder_kernels.utils.dochelpers import (getargspecfromtext, getobj,
 
 # Local imports
 from spyder.config.manager import CONF
-from spyder.py3compat import is_text_string, to_text_string
+from spyder.py3compat import to_text_string
 from spyder.utils import encoding, sourcecode
 from spyder.utils import syntaxhighlighters as sh
 from spyder.utils.misc import get_error_match
@@ -682,18 +684,35 @@ class BaseEditMixin(object):
         pass
 
     #------EOL characters
-    def set_eol_chars(self, text):
-        """Set widget end-of-line (EOL) characters from text (analyzes text)"""
-        if not is_text_string(text): # testing for QString (PyQt API#1)
-            text = to_text_string(text)
-        eol_chars = sourcecode.get_eol_chars(text)
-        is_document_modified = eol_chars is not None and self.eol_chars is not None
-        self.eol_chars = eol_chars
+    def set_eol_chars(self, text=None, eol_chars=None):
+        """
+        Set widget end-of-line (EOL) characters.
+
+        Parameters
+        ----------
+        text: str
+            Text to detect EOL characters from.
+        eol_chars: str
+            EOL characters to set.
+
+        Notes
+        -----
+        If `text` is passed, then `eol_chars` has no effect.
+        """
+        if text is not None:
+            detected_eol_chars = sourcecode.get_eol_chars(text)
+            is_document_modified = (
+                detected_eol_chars is not None and self.eol_chars is not None
+            )
+            self.eol_chars = detected_eol_chars
+        elif eol_chars is not None:
+            is_document_modified = eol_chars != self.eol_chars
+            self.eol_chars = eol_chars
+
         if is_document_modified:
             self.document().setModified(True)
             if self.sig_eol_chars_changed is not None:
                 self.sig_eol_chars_changed.emit(eol_chars)
-            self.document_did_change(text)
 
     def get_line_separator(self):
         """Return line separator based on current EOL mode"""
@@ -858,7 +877,6 @@ class BaseEditMixin(object):
         """
         self.__move_cursor_anchor(what, direction, QTextCursor.MoveAnchor)
 
-
     #------Selection
     def extend_selection_to_next(self, what='word', direction='left'):
         """
@@ -867,9 +885,10 @@ class BaseEditMixin(object):
         """
         self.__move_cursor_anchor(what, direction, QTextCursor.KeepAnchor)
 
-
     #------Text: get, set, ...
-    def __select_text(self, position_from, position_to):
+
+    def _select_text(self, position_from, position_to):
+        """Select text and return cursor."""
         position_from = self.get_position(position_from)
         position_to = self.get_position(position_to)
         cursor = self.textCursor()
@@ -912,7 +931,7 @@ class BaseEditMixin(object):
         TODO: Evaluate if this is still a problem and if the workaround can
               be moved closer to where the problem occurs.
         """
-        cursor = self.__select_text(position_from, position_to)
+        cursor = self._select_text(position_from, position_to)
         text = to_text_string(cursor.selectedText())
         if remove_newlines:
             remove_newlines = position_from != 'sof' or position_to != 'eof'
@@ -942,10 +961,9 @@ class BaseEditMixin(object):
             self.textCursor().insertText(text)
             if self.sig_text_was_inserted is not None:
                 self.sig_text_was_inserted.emit()
-            self.document_did_change()
 
     def replace_text(self, position_from, position_to, text):
-        cursor = self.__select_text(position_from, position_to)
+        cursor = self._select_text(position_from, position_to)
         if self.sig_will_remove_selection is not None:
             start, end = self.get_selection_start_end(cursor)
             self.sig_will_remove_selection.emit(start, end)
@@ -955,15 +973,13 @@ class BaseEditMixin(object):
         cursor.insertText(text)
         if self.sig_text_was_inserted is not None:
             self.sig_text_was_inserted.emit()
-        self.document_did_change()
 
     def remove_text(self, position_from, position_to):
-        cursor = self.__select_text(position_from, position_to)
+        cursor = self._select_text(position_from, position_to)
         if self.sig_will_remove_selection is not None:
             start, end = self.get_selection_start_end(cursor)
             self.sig_will_remove_selection.emit(start, end)
         cursor.removeSelectedText()
-        self.document_did_change()
 
     def get_current_object(self):
         """
@@ -1180,12 +1196,14 @@ class BaseEditMixin(object):
         """Delete selected text."""
         self.textCursor().removeSelectedText()
         # The next three lines are a workaround for a quirk of
-        # QTextEdit. See spyder-ide/spyder#12663 and
+        # QTextEdit on Linux with Qt < 5.15, MacOs and Windows.
+        # See spyder-ide/spyder#12663 and
         # https://bugreports.qt.io/browse/QTBUG-35861
-        cursor = self.textCursor()
-        cursor.setPosition(cursor.position())
-        self.setTextCursor(cursor)
-        self.document_did_change()
+        if (parse_version(QT_VERSION) < parse_version('5.15')
+                or os.name == 'nt' or sys.platform == 'darwin'):
+            cursor = self.textCursor()
+            cursor.setPosition(cursor.position())
+            self.setTextCursor(cursor)
 
     def replace(self, text, pattern=None):
         """Replace selected text by *text*.
@@ -1209,7 +1227,6 @@ class BaseEditMixin(object):
         if self.sig_text_was_inserted is not None:
             self.sig_text_was_inserted.emit()
         cursor.endEditBlock()
-        self.document_did_change()
 
 
     #------Find/replace
@@ -1394,7 +1411,6 @@ class BaseEditMixin(object):
                 if self.sig_text_was_inserted is not None:
                     self.sig_text_was_inserted.emit()
                 cursor.endEditBlock()
-                self.document_did_change()
 
 
 class TracebackLinksMixin(object):
